@@ -165,12 +165,13 @@ gen_ttyrec_filename ()
 char*
 gen_inprogress_lock (pid_t pid, char* ttyrec_filename)
 {
-  char *lockfile = NULL, pidbuf[16];
+  char *lockfile = NULL, filebuf[80];
   int fd;
   size_t len;
   struct flock fl = { 0 };
 
-  snprintf (pidbuf, 16, "%d", pid);
+  snprintf (filebuf, sizeof(filebuf), "%d\n%d\n%d\n",
+		  pid, win.ws_row, win.ws_col);
 
   fl.l_type = F_WRLCK;
   fl.l_whence = SEEK_SET;
@@ -187,7 +188,7 @@ gen_inprogress_lock (pid_t pid, char* ttyrec_filename)
   if (fcntl (fd, F_SETLKW, &fl) == -1)
     graceful_exit (68);
 
-  write (fd, pidbuf, strlen (pidbuf));
+  write (fd, filebuf, strlen (filebuf));
 
   return lockfile;
 }
@@ -305,10 +306,11 @@ drawbanner (unsigned int start_line, unsigned int howmany)
 void
 inprogressmenu ()
 {
-  int i, menuchoice, len = 20, offset = 0;
+  int i, menuchoice, len = 20, offset = 0, doresizewin = 0;
   time_t ctime;
   struct dg_game **games;
   char ttyrecname[130], *replacestr = NULL;
+  sigset_t oldmask, toblock;
 
   games = populate_games (&len);
 
@@ -320,7 +322,7 @@ inprogressmenu ()
                 "During playback, hit 'q' to return here, 'm' to send mail (requires login),");
       mvaddstr (4, 1,
                 "'s' to toggle graphic-set stripping for DEC, IBM, and none.");
-      mvaddstr (5, 1, "The following games are in progress:");
+      mvaddstr (5, 1, "The following games are in progress: (use uppercase to try to change size)");
 
       /* clean old games and list good ones */
       i = 0;
@@ -330,18 +332,21 @@ inprogressmenu ()
           if (i + offset >= len)
             break;
 
-          mvprintw (7 + i, 1, "%c) %-15s %s %s (%ldm %lds idle)",
+          mvprintw (7 + i, 1, "%c) %-15s (%3dx%3d) %s %s (%ldm %lds idle)",
                     i + 97, games[i + offset]->name,
+		    games[i + offset]->ws_col, games[i + offset]->ws_row,
                     games[i + offset]->date, games[i + offset]->time,
                     (time (&ctime) - games[i + offset]->idle_time) / 60,
                     (time (&ctime) - games[i + offset]->idle_time) % 60);
         }
 
+      if (len > 0)
+        mvprintw (21, 1, "(%d-%d of %d)", offset + 1, offset + i, len);
       mvaddstr (23, 1,
                 "Watch which game? (any key refreshes, 'q' quits, '>'/'<' for more/less) => ");
       refresh ();
 
-      switch ((menuchoice = tolower (getch ())))
+      switch ((menuchoice = getch ()))
         {
         case '>':
           if ((offset + 14) >= len)
@@ -357,11 +362,17 @@ inprogressmenu ()
             offset -= 14;
           break;
 
-        case 'q':
+	case 'q': case 'Q':
           return;
 
         default:
-          if ((menuchoice - 97) >= 0 && (menuchoice - 97) < i)
+	  doresizewin = 0;
+	  if (isupper (menuchoice))
+	    {
+	      doresizewin = 1;
+	      menuchoice = tolower (menuchoice);
+	    }
+          if ((menuchoice - 'a') >= 0 && (menuchoice - 'a') < i)
             {
               /* valid choice has been made */
               snprintf (ttyrecname, 130, "%sttyrec/%s", myconfig->dglroot,
@@ -379,8 +390,24 @@ inprogressmenu ()
               clear ();
               refresh ();
               endwin ();
+	      if (doresizewin)
+	        {
+		  /*
+		   * Let curses deal with the resize later. Perhaps this is
+		   * not the best way.
+		   */
+		  sigemptyset (&toblock);
+		  sigaddset (&toblock, SIGWINCH);
+		  sigprocmask (SIG_BLOCK, &toblock, &oldmask);
+		  printf ("\033[8;%d;%dt",
+		    games[menuchoice - 97 + offset]->ws_row,
+		    games[menuchoice - 97 + offset]->ws_col);
+		  fflush (stdout);
+		}
               ttyplay_main (ttyrecname, 1);
               initcurses ();
+	      if (doresizewin)
+	        sigprocmask (SIG_SETMASK, &oldmask, NULL);
             }
         }
 
@@ -390,6 +417,47 @@ inprogressmenu ()
 
 /* ************************************************************* */
 
+/*
+ * Check email address, returns 1 if valid, 0 otherwise.
+ * Doesn't recognize addresses with parts in double-quotes.
+ * Addresses with a colon in them are always rejected.
+ */
+int
+check_email (char *s)
+{
+  char *atomchars = "!#$%&'*+-/=?^_`{|}~" "0123456789"
+    "abcdefghijklmnopqrstuvwxyz" "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+  int f;
+
+  if (*s == '@')
+    return 0;
+
+  while (*s != '\0' && *s != '@')
+    {
+      if (strchr(atomchars, *s) == NULL)
+        return 0;
+      s++;
+      if (*s == '.')
+        s++;
+    }
+
+  if (*s == '\0')
+    return 0;
+  s++;
+
+  f = 0;
+  while (*s != '\0')
+    {
+      if (strchr(atomchars, *s) == NULL)
+        return 0;
+      s++;
+      if (*s == '.')
+        s++, f = 1;
+    }
+
+  return f;
+}
+
 void
 change_email ()
 {
@@ -415,8 +483,7 @@ change_email ()
       mvaddstr (8, 1, "That's the same one as before. Try again?");
       move(1,1);
     }
-    /* rudimentary check for validity */
-    else if (strchr(buf, '@') < strrchr(buf, '.'))
+    else if (check_email (buf))
     {
       mvprintw (8, 1, "Changing email address to '%s'. Confirm (y/n): ", buf);
       if (getch() == 'y')
@@ -834,32 +901,45 @@ newuser ()
 
   /* email step */
 
-  clear ();
+  error = 2;
+  while (error != 0)
+    {
+      clear ();
 
-  drawbanner (1, 1);
+      drawbanner (1, 1);
 
-  mvaddstr (5, 1, "Please enter your email address.");
-  mvaddstr (6, 1,
-            "This is sent _nowhere_ but will be used if you ask the sysadmin for lost");
-  mvaddstr (7, 1,
-            "password help. Please use a correct one. It only benefits you.");
-  mvaddstr (8, 1, "80 character max. No ':' characters. Blank line aborts.");
-  mvaddstr (10, 1, "=> ");
+      mvaddstr (5, 1, "Please enter your email address.");
+      mvaddstr (6, 1, "This is sent _nowhere_ but will be used if you ask"
+        " the sysadmin for lost");
+      mvaddstr (7, 1, "password help. Please use a correct one. It only"
+        " benefits you.");
+      mvaddstr (8, 1, "80 character max. No ':' characters. Blank line"
+        " aborts.");
+      mvaddstr (10, 1, "=> ");
 
-  refresh ();
-  mygetnstr (buf, 80, 1);
+      if (error == 1)
+        {
+          mvaddstr (12, 1, "There was a problem with your last entry.");
+          move (10, 4);
+        }
 
-  if (strchr (buf, ':') != NULL)
-    graceful_exit (113);
-
-  if (buf && *buf == '\0')
-  {
-    free (me->username);
-    free (me->password);
-    free (me);
-    me = NULL;
-    return;
-  }
+      refresh ();
+      mygetnstr (buf, 80, 1);
+ 
+      if (check_email (buf))
+        error = 0;
+      else
+        error = 1;
+ 
+      if (buf && *buf == '\0')
+      {
+        free (me->username);
+        free (me->password);
+        free (me);
+        me = NULL;
+        return;
+      }
+    }
 
   me->email = strdup (buf);
   me->env = calloc (1, 1);
@@ -1411,10 +1491,14 @@ main (int argc, char** argv)
   signal (SIGHUP, catch_sighup);
 
   (void) tcgetattr (0, &tt);
-  win.ws_row = 24;
-  win.ws_col = 80;
-  win.ws_xpixel = win.ws_col * 8;
-  win.ws_ypixel = win.ws_row * 8;
+  if (-1 == ioctl (0, TIOCGWINSZ, (char *) &win) || win.ws_row < 4 ||
+		  win.ws_col < 4) /* Rudimentary validity check */
+    {
+      win.ws_row = 24;
+      win.ws_col = 80;
+      win.ws_xpixel = win.ws_col * 8;
+      win.ws_ypixel = win.ws_row * 8;
+    }
 
   /* get master tty just before chroot (lives in /dev) */
   ttyrec_getpty ();
