@@ -352,7 +352,7 @@ populate_gfx_array (int gfxset)
   for (i = 0; i < MAXPCHARS; i++)
     {
       if ((gfxset == DEC_GRAPHICS) && (dec_graphics[i])
-          && !(gfx_map[dec_graphics[i] - 128]))
+          && !(gfx_map[dec_graphics[i]]))
         gfx_map[dec_graphics[i] - 128] = no_graphics[i];
       if ((gfxset == IBM_GRAPHICS) && (ibm_graphics[i]))
         gfx_map[ibm_graphics[i]] = no_graphics[i];
@@ -372,45 +372,21 @@ populate_gfx_array (int gfxset)
 unsigned char
 strip_gfx (unsigned char inchar)
 {
-
-  if (state == 0)
+  if ((inchar == 0x0E) && (state == 0))
     {
-      if (inchar == 0x0E)
-        {
-	  state = 1;
-	  return 0x00;
-	}
-      else if (inchar == 0x1B)
-        {
-	  state = 2;
-	  return inchar;
-	}
+      state = 1;
+      return 0x00;
     }
 
-  if (state == 1)
+  if ((inchar == 0x0F) && (state == 1))
     {
-      if (inchar == 0x0F || inchar == 0x1B)
-        {
-	  state = 0;
-	  return inchar;
-	}
-    }
-
-  if (state == 2)
-    {
-      if (inchar == '(')
-        state = 3;
-      else
-        state = 0;
+      state = 0;
       return inchar;
     }
 
-  if (state == 3)
+  if ((inchar == 0x1B) && (state == 1))
     {
-      if (inchar == '0') /* ^[(0 */
-        state = 1;
-      else /* ^[(B or something similar */
-        state = 0;
+      state = 0;
       return inchar;
     }
 
